import threading


class Test:
    def __init__(self):
        self.a = "a"
        self.T = threading.Thread(target=self.my_method, args=["zzz"])
        
    def my_method(self, x):
        print("Hole: {}".format(x))
        
    def start(self):
        self.T.start()
        

T = Test()
T.start()




#!/usr/bin/env python3
"""GUI to upload tests."""
import argparse
import json
import sys
from pathlib import Path

try:
    import itkdb_gtk

except ImportError:
    cwd = Path(__file__).parent.parent
    sys.path.append(cwd.as_posix())

from itkdb_gtk import dbGtkUtils, ITkDBlogin, ITkDButils, QRScanner
from itkdb_gtk.GetShipments import find_vtrx


import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio

class FindComponent(dbGtkUtils.ITkDBWindow):
    """Read QR of bar code and retrieve information about component."""

    def __init__(self, session, help_link=None):
        """Initialization.

        Args:
            session: ITkDB session
            help_link: link to help page.

        """
        super().__init__(session=session, title="Find Component", help_link=help_link)
        self.scanner = QRScanner.QRScanner(self.get_qrcode)
        self.init_window()

    def init_window(self):
        """Create the Gtk window."""
        # Initial tweaks
        self.set_border_width(10)

        # Prepare HeaderBar
        self.hb.props.title = "Find Component"

        # Object Data
        lbl = Gtk.Label(label="Scan your QR or bar code. Information will appear below.")
        self.mainBox.pack_start(lbl, False, False, 10)

        #btn = Gtk.Button(label="Test Button")
        #btn.connect("clicked", self.test_qrcode)
        #self.mainBox.pack_start(btn, True, True, 0)

        # The text view
        self.mainBox.pack_start(self.message_panel.frame, True, True, 0)

        self.show_all()

    def get_qrcode(self, txt):
        """Gets data from QR scanner."""
        if txt.find("J-SD") == 0:
            try:
                SN = find_vtrx(self.session, txt)
            except ValueError as e:
                self.write_message("Error: {}\n".format(e))
                return
        else:
            SN = txt

        obj = ITkDButils.get_DB_component(self.session, SN)
        if obj is None:
            self.write_message("Object not found in DB\n")
            return

        msg = "\n{}\nObject SN: {}\nObject Alt. ID: {}\nObject Type: {}\nObject Loc. {}\nObject stage: {} - {}\n".format(
            txt,
            obj["serialNumber"],
            obj["alternativeIdentifier"],
            obj["componentType"]["name"],
            obj["currentLocation"]["name"],
            obj["currentStage"]["code"],
            obj["currentStage"]["name"])

        self.write_message(msg)

    def test_qrcode(self, *args):
        """Gets data from QR scanner."""
        txt = "a3c671bf38d3957dc053c6e5471aa27e"
        self.write_message("{}\n".format(txt))

        if txt.find("J-SD") == 0:
            try:
                SN = find_vtrx(self.session, txt)
            except ValueError as e:
                self.write_message("Error: {}\n".format(e))
                return
        else:
            SN = txt

        obj = ITkDButils.get_DB_component(self.session, SN)
        if obj is None:
            self.write_message("Object not found in DB\n")
            return


        msg = "\n\nObject SN: {}\nObject Alt. ID: {}\nObject Type: {}\nObject Loc.: {}\nObject stage: {} - {}\n".format(
            obj["serialNumber"],
            obj["alternativeIdentifier"],
            obj["componentType"]["name"],
            obj["currentLocation"]["name"],
            obj["currentStage"]["code"],
            obj["currentStage"]["name"])

        self.write_message(msg)
        self.write_message("")

def main():
    """Main entry."""
    HELP_LINK="https://itkdb-gtk.docs.cern.ch/uploadSingleTest.html"

    # DB login
    dlg = ITkDBlogin.ITkDBlogin()
    client = dlg.get_client()
    if client is None:
        print("Could not connect to DB with provided credentials.")
        dlg.die()
        sys.exit()

    client.user_gui = dlg

    window = FindComponent(client, help_link=HELP_LINK)
    window.set_accept_focus(True)
    window.present()
    window.connect("destroy", Gtk.main_quit)

    try:
        Gtk.main()

    except KeyboardInterrupt:
        print("Arrrgggg!!!")

    dlg.die()

if __name__ == "__main__":
    main()


#!/usr/bin/env python3
"""Tests the range parser."""
def range_to_list(V):
    """Convert a range to a list.

        ch1-ch2 or ch1:ch2 -> [ch1, ch1+1, ch1+2, ..., ch2] or
        ch1:step:ch2 -> [ch1, ch1+sep, ch1+2*step, ..., ch2]
    """
    nfound = 0
    for c in "-:,":
        if c in V:
            nfound += 1
            break

    if nfound == 0:
        return [V]

    out = []
    values = V.split(',')
    for V in values:
        if '-' in V:
            endpoints = list(map(int, V.split('-')))
            endpoints.sort()
            for i in range(endpoints[0], endpoints[1]+1):
                out.append(str(i))
        elif ':' in V:
            endpoints = list(map(int, V.split(':')))
            if len(endpoints) == 2:
                endpoints.sort()
                for i in range(endpoints[0], endpoints[1]+1):
                    out.append(str(i))

            elif len(endpoints) == 3:
                for i in range(endpoints[0], endpoints[2]+1, endpoints[1]):
                    out.append(str(i))

            else:
                print("Wring range specification. {}".format(V))
                continue

        else:
            out.append(V)

    return out

def main():
    r = "1-5"
    print(r, range_to_list(r))

    r = "1:5"
    print(r, range_to_list(r))

    r = "1:2:12"
    print(r, range_to_list(r))


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""GUI to upload tests."""
import fnmatch
import json
import os
import sys
from pathlib import Path

try:
    import itkdb_gtk

except ImportError:
    cwd = Path(__file__).parent.parent
    sys.path.append(cwd.as_posix())

from itkdb_gtk import dbGtkUtils, ITkDBlogin, ITkDButils
from itkdb_gtk.ShowComments import ShowComments
from itkdb_gtk.ShowAttachments import ShowAttachments
from itkdb_gtk.ShowDefects import ShowDefects
from itkdb_gtk.UploadTest import create_json_data_editor

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio, Gdk

# Check if Gtk can be open
gtk_runs, gtk_args = Gtk.init_check()


def handle_test_date(the_date):
    """Edit date."""
    the_date = the_date[:19].replace('T', ' ')
    return the_date


def all_files(root, patterns='*', single_level=False, yield_folders=False):
    """A generator that reruns all files in the given folder.

    Args:
        root (file path): The folder
        patterns (str, optional): The pattern of the files. Defaults to '*'.
        single_level (bool, optional): If true, do not go into sub folders. Defaults to False.
        yield_folders (bool, optional): If True, return folders as well. Defaults to False.

    Yields:
        str: file path name

    """
    patterns = patterns.split(';')
    for path, subdirs, files in os.walk(root):
        if yield_folders:
            files.extend(subdirs)

        files.sort()
        for name in files:
            for pattern in patterns:
                if fnmatch.fnmatch(name, pattern):
                    yield os.path.join(path, name)
                    break

        if single_level:
            break


class TestList(object):
    """Enumeration with TreeView model columns."""
    (
        SN,
        TestType,
        RunNumber,
        Date,
        Institute,
        Stage,
        currentStage,
        Path,
        Json,
        Nattch,
        Attachments,
        Ncomm,
        Comments,
        Ndef,
        Defects,
        Color,
        ALL,
    ) = range(17)


def check_data(data):
    """Checks validity of JSon data.

    Args:
        data (): The json data

    Returns:
    -------
        boolean: True if valid, False otherwise.

    """
    errors = []
    missing = []
    if "component" not in data:
        errors.append("Need reference to component, hex string")
        missing.append("component")

    if "testType" not in data:
        errors.append("Need to know test type, short code")
        missing.append("testType")

    if "institution" not in data:
        errors.append("Need to know institution, short code")
        missing.append("institution")

    if "results" not in data:
        errors.append("Need some test results")
        missing.append("results")

    return errors, missing


class UploadMultipleTests(dbGtkUtils.ITkDBWindow):
    """Collects information to upload a test and its attachments."""

    def __init__(self, session, help_link=None):
        """Initialization.

        Args:
            session: ITkDB session

        """
        super().__init__(session=session, title="Upload Tests", gtk_runs=gtk_runs, help_link=help_link)
        self.tests = []
        self.data = None
        self.tree = None
        self.is_retroactive = False
        self.def_color = None

        self.init_window()

    def init_window(self):
        """Creates the Gtk window."""
        # Initial tweaks
        self.set_border_width(10)

        # Prepare HeaderBar
        self.hb.props.title = "Upload Multiple Tests"

        # Active buttin in header
        button = Gtk.Button()
        icon = Gio.ThemedIcon(name="document-send-symbolic")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        button.add(image)
        button.set_tooltip_text("Click to upload test")
        button.connect("clicked", self.upload_test_gui)
        self.hb.pack_end(button)

        # Data panel
        grid = Gtk.Grid(column_spacing=5, row_spacing=1)
        self.mainBox.pack_start(grid, False, False, 0)

        # The test file widgets
        lbl = Gtk.Label(label="Select Test Files: ")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, 0, 1, 1)

        btn = Gtk.Button()
        icon = Gio.ThemedIcon(name="text-x-generic-symbolic")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        btn.add(image)
        btn.set_tooltip_text("Click to select multiple tests.")
        btn.connect("clicked", self.on_select_test)
        grid.attach(btn, 1, 0, 1, 1)

        btn = Gtk.Button()
        icon = Gio.ThemedIcon(name="folder-symbolic")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        btn.add(image)
        btn.set_tooltip_text("Click to select a folder to scan.")
        btn.connect("clicked", self.on_select_folder)
        grid.attach(btn, 2, 0, 1, 1)

        # toggle = Gtk.ToggleButton("RetroActive")
        # toggle.set_tooltip_text("Toggle to make all uploads retroactive.")
        # toggle.connect("toggled", self.on_toggle_retroactive)
        # grid.attach(toggle, 3, 0, 1, 1)

        # Paned object
        paned = Gtk.Paned(orientation=Gtk.Orientation.VERTICAL)
        paned.set_size_request(-1, 200)
        self.mainBox.pack_start(paned, True, True, 5)

        # the list of attachments
        tree_view = self.create_tree_view()
        paned.add1(tree_view)

        # The text view
        paned.add2(self.message_panel.frame)

        self.show_all()

    def create_tree_view(self, size=150):
        """Creates the tree vew with the attachments."""
        model = Gtk.ListStore(str, # SN
                              str, # test type
                              str, # runNumber
                              str, # date
                              str, # institute
                              str, # stage
                              str, # stage
                              str, # ifile
                              object, # data
                              int, # num. attch.
                              object, # attachments
                              int, # num. comments
                              object, # comments
                              int, # num defects
                              object, # defects
                              str # color
                              )
        self.tree = Gtk.TreeView(model=model)
        self.tree.connect("button-press-event", self.button_pressed)
        scrolled = Gtk.ScrolledWindow()
        scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        scrolled.add(self.tree)
        scrolled.set_size_request(-1, size)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("SN", renderer, text=TestList.SN)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        self.def_color = renderer.get_property("foreground-rgba").to_string()
        column = Gtk.TreeViewColumn("Test Type", renderer, text=TestList.TestType, foreground=TestList.Color)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Run", renderer, text=TestList.RunNumber)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Institute", renderer, text=TestList.Institute)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Stage", renderer, text=TestList.Stage)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("N. att.", renderer, text=TestList.Nattch)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("N. comm.", renderer, text=TestList.Ncomm)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("N. def.", renderer, text=TestList.Ndef)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Date", renderer, text=TestList.Date)
        self.tree.append_column(column)

        return scrolled

    def button_pressed(self, tree, event):
        """Button pressed on tree view."""
        # double click shows attachments
        if event.button == 1 and event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS:
            select = self.tree.get_selection()
            model, lv_iter = select.get_selected()
            if not lv_iter:
                return

            self.on_show_json(None, (model, lv_iter, model[lv_iter]))
            # self.on_show_attachments(None, (model, lv_iter, model[lv_iter]))
            return

        if event.button != 3:
            return

        # Create popup menu
        select = self.tree.get_selection()
        model, lv_iter = select.get_selected()
        values = None
        if lv_iter:
            values = model[lv_iter]

        if not lv_iter:
            P = tree.get_path_at_pos(event.x, event.y)
            if P:
                print(P[0].to_string())
                lv_iter = model.get_iter(P[0])
                values = model[lv_iter]

        if not values:
            return

        menu = Gtk.Menu()
        item_show = Gtk.MenuItem(label="Show JSOn")
        item_show.connect("activate", self.on_show_json, (model, lv_iter, values))
        menu.append(item_show)

        item_set_stage = Gtk.MenuItem(label="Set Stage")
        item_set_stage.connect("activate", self.on_set_stage, (model, lv_iter, values))
        menu.append(item_set_stage)

        item_show_att = Gtk.MenuItem(label="Edit Attachments")
        item_show_att.connect("activate", self.on_show_attachments, (model, lv_iter, values))
        menu.append(item_show_att)

        item_show_com = Gtk.MenuItem(label="Edit Comments")
        item_show_com.connect("activate", self.on_show_comments, (model, lv_iter, values))
        menu.append(item_show_com)

        item_show_def = Gtk.MenuItem(label="Edit Defects")
        item_show_def.connect("activate", self.on_show_defects, (model, lv_iter, values))
        menu.append(item_show_def)

        item_del = Gtk.MenuItem(label="Delete")
        item_del.connect("activate", self.on_delete_tests, (model, lv_iter, values))
        menu.append(item_del)
        menu.show_all()

        menu.popup_at_pointer(event)

    def on_toggle_retroactive(self, toggle, *args):
        """Called when retroactive button is toggled."""
        self.is_retroactive = toggle.get_active()
        print("Is retroactive", self.is_retroactive)

    def on_show_json(self, item, data):
        """Test JSon."""
        model, lv_iter, val = data
        payload = val[TestList.Json]
        value, dlg = create_json_data_editor(payload)
        rc = dlg.run()
        if rc == Gtk.ResponseType.OK:
            payload = value.values
            model.set_value(lv_iter, TestList.Json, payload)
            model.set_value(lv_iter, TestList.SN, payload["component"])
            model.set_value(lv_iter, TestList.RunNumber, payload["runNumber"])
            model.set_value(lv_iter, TestList.Date, handle_test_date(payload["date"]))
            model.set_value(lv_iter, TestList.Institute, handle_test_date(payload["institution"]))

        dlg.hide()
        dlg.destroy()

    def on_show_attachments(self, item, data):
        """Show the attachmetns."""
        model, lv_iter, val = data

        SA = ShowAttachments("Test Attachments", self.session, val[TestList.Attachments], parent=self)
        response = SA.run()
        if response == Gtk.ResponseType.OK:
            model.set_value(lv_iter, TestList.Attachments, SA.attachments)
            model.set_value(lv_iter, TestList.Nattch, len(SA.attachments))

        SA.hide()
        SA.destroy()

    def on_show_comments(self, item, data):
        """Show comments"""
        model, lv_iter, val = data
        SC = ShowComments("Test Comments", val[TestList.Comments], self)
        rc = SC.run()
        if rc == Gtk.ResponseType.OK:
            model.set_value(lv_iter, TestList.Comments, SC.comments)
            model.set_value(lv_iter, TestList.Ncomm, len(SC.comments))

        SC.hide()
        SC.destroy()

    def on_show_defects(self, item, data):
        """Show comments"""
        model, lv_iter, val = data
        SD = ShowDefects("Test Defects", val[TestList.Defects], self)
        rc = SD.run()
        if rc == Gtk.ResponseType.OK:
            model.set_value(lv_iter, TestList.Defects, SD.defects)
            model.set_value(lv_iter, TestList.Ndef, len(SD.defects))

        SD.hide()
        SD.destroy()

    def on_delete_tests(self, item, data):
        """Test edit."""
        model, lv_iter, val = data
        rc = dbGtkUtils.ask_for_confirmation("Remove this test?",
                                             "{} - {}".format(val[TestList.SN], val[TestList.TestType]))
        if rc:
            model.remove(lv_iter)

    def get_test_institute(self):
        """Select an institue."""
        dlg = Gtk.Dialog(title="Select Institution.", flags=0)
        dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                        Gtk.STOCK_OK, Gtk.ResponseType.OK)
        area = dlg.get_content_area()
        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        area.add(box)

        box.pack_start(Gtk.Label(label="Select an Institute"), False, True, 0)

        combo = self.create_institute_combo(only_user=True)
        box.pack_start(combo, False, True, 5)

        btn = Gtk.CheckButton(label="Use as default for other tests missing institute ?")
        box.pack_start(btn, False, True, 5)
        dlg.show_all()
        rc = dlg.run()

        out = None
        if rc == Gtk.ResponseType.OK:
            out = self.get_institute_from_combo(combo)

        use_default = btn.get_active()
        dlg.hide()
        dlg.destroy()
        return out, use_default

    def on_set_stage(self, item, data):
        """Set the test stage."""
        model, lv_iter, val = data
        SN = val[TestList.SN]
        combo, _ = self.get_component_stages(SN)

        dlg = Gtk.Dialog(title="Set object stage")

        dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                        Gtk.STOCK_OK, Gtk.ResponseType.OK)
        area = dlg.get_content_area()
        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        area.add(box)

        box.pack_start(Gtk.Label(label="Select Stage"), False, True, 0)
        box.pack_start(combo, False, True, 0)
        toggle = Gtk.ToggleButton("Retroactive")
        box.pack_start(toggle, False, True, 0)

        dlg.show_all()

        rc = dlg.run()
        if rc == Gtk.ResponseType.OK:
            new_stage = combo.get_active_text()
            model[lv_iter][TestList.Stage] = new_stage
            data = model[lv_iter][TestList.Json]
            if toggle.get_active():
                data["isRetroactive"] = True
                data["stage"] = new_stage

            else:
                changed = (new_stage != model[lv_iter][TestList.currentStage])
                data["isRetroactive"] = changed
                if changed:
                    data["stage"] = new_stage
                else:
                    if "stage" in data:
                        del data["stage"]

            model[lv_iter][TestList.Json] = data

        dlg.hide()
        dlg.destroy()

    def get_component_stages(self, SN):
        """Create a combo with the stages."""
        try:
            obj = ITkDButils.get_DB_component(self.session, SN)
            currentStage = obj["currentStage"]["code"]

            combo = Gtk.ComboBoxText.new_with_entry()
            combo.remove_all()
            indx = 0
            for i, stg in enumerate(obj["stages"]):
                S = stg["code"]
                combo.append_text(S)
                if S == currentStage:
                    indx = i

            combo.set_active(indx)
            return combo, currentStage

        except Exception:
            self.write_message("Something went wrong with the stages\n")
            return [None, None]

    def add_test_data_to_view(self, data, default_site=None, use_default=False, ifile=None, folder=None):
        """Add a test data to the  tree view."""
        has_errors = False
        errors, missing = check_data(data)
        if len(missing) > 0:
            self.write_message("Some keys are missing in the JSon file.\n")
            self.write_message("{}\n".format("\n".join(['\t'+line for line in missing])))

            if "institution" in missing and len(missing) == 1:
                if default_site is None:
                    site, use_default = self.get_test_institute()
                    if use_default:
                        default_site = site
                else:
                    site = default_site

                if site:
                    data["institution"] = site
                    self.write_message("Setting Institution to {}\n".format(data["institution"]))

            else:
                has_errors = True
                dbGtkUtils.complain("Invalid JSON file\n{}".format('\n'.join(errors)),"--")

        if not has_errors:
            attachments = []
            if "attachments" in data:
                for att in data["attachments"]:
                    path = Path(att["path"])
                    if path.exists():
                        path = path.expanduser().resolve()
                    else:
                        if folder is not None:
                            path = folder / path.name

                    if path.exists():
                        attachments.append(ITkDButils.Attachment(path=path,
                                                                 title=att["title"],
                                                                 desc=att["description"]))
                    else:
                        self.write_message("Ignoring atachment {}".format(data["path"]))

                # We need to delete this, which is "unofficial"
                del data["attachments"]

            model = self.tree.get_model()
            comments = data.get("comments", [])
            defects = data.get("defects", [])
            the_date = handle_test_date(data["date"])
            combo, currentStage = self.get_component_stages(data["component"])
            if data["passed"]:
                if data["problems"]:
                    color = "orange"
                else:
                    color = self.def_color
            else:
                color = "firebrick"

            model.append([data["component"], data["testType"], data["runNumber"], the_date,
                            data["institution"], currentStage, currentStage,
                            ifile, data, len(attachments), attachments,
                            len(comments), comments, len(defects), defects, color])

        return default_site, use_default

    def add_tests_to_view(self, files):
        """Add the input fiels to the treeview."""
        default_site = None
        use_default = False
        for ifile in files:
            try:
                self.write_message("{}\n".format(Path(ifile).name))
                folder = Path(ifile).parent

                data = json.loads(open(ifile, "r", encoding="UTF-8").read())
                default_site, use_default = self.add_test_data_to_view(
                    data,
                    default_site=default_site,
                    use_default=use_default,
                    ifile=ifile,
                    folder=folder,
                )

            except Exception as E:
                self.write_message("Cannot load file {}\n".format(ifile))
                self.write_message("{}\n".format(str(E)))

    def on_select_folder(self, *args):
        """Caalback for select folder button"""
        fdlg = Gtk.FileChooserNative(action=Gtk.FileChooserAction.SELECT_FOLDER, accept_label="Select")
        response = fdlg.run()
        if response == Gtk.ResponseType.ACCEPT:
            folder = fdlg.get_filename()
            ifiles = [ipath for ipath in all_files(folder, '*.json')]
            self.add_tests_to_view(ifiles)

        fdlg.hide()
        fdlg.destroy()

    def on_select_test(self, *args):
        """Test file browser clicked."""
        fdlg = Gtk.FileChooserNative(action=Gtk.FileChooserAction.OPEN, accept_label="Select")

        filter_js = Gtk.FileFilter()
        filter_js.set_name("JSon files")
        filter_js.add_mime_type("application/json")
        fdlg.add_filter(filter_js)

        filter_any = Gtk.FileFilter()
        filter_any.set_name("Any files")
        filter_any.add_pattern("*")
        fdlg.add_filter(filter_any)

        fdlg.set_select_multiple(True)

        response = fdlg.run()
        if response == Gtk.ResponseType.ACCEPT:
            ifiles = [ipath for ipath in fdlg.get_filenames()]
            self.add_tests_to_view(ifiles)

        fdlg.hide()
        fdlg.destroy()
        return

    def show_data(self, *args):
        """Show data button clicked."""
        if self.data is None:
            return

        dlg = Gtk.Dialog(title="Test Data")
        dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                        Gtk.STOCK_OK, Gtk.ResponseType.OK)

        dlg.set_property("height-request", 500)
        box = dlg.get_content_area()
        value = dbGtkUtils.DictDialog(self.data)
        scrolled = Gtk.ScrolledWindow()
        scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        scrolled.add(value)
        box.pack_start(scrolled, True, True, 10)

        dlg.show_all()

        rc = dlg.run()
        if rc == Gtk.ResponseType.OK:
            self.data = value.values

        dlg.hide()
        dlg.destroy()

    def add_attachment_dialog(self):
        """Create the add attachment dialog."""
        dlg = Gtk.Dialog(title="Add Attachment")
        dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                        Gtk.STOCK_OK, Gtk.ResponseType.OK)
        grid = Gtk.Grid(column_spacing=5, row_spacing=1)
        box = dlg.get_content_area()
        box.add(grid)

        lbl = Gtk.Label(label="File")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, 0, 1, 1)

        lbl = Gtk.Label(label="Title")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, 1, 1, 1)

        lbl = Gtk.Label(label="Description")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, 2, 1, 1)

        dlg.fC = Gtk.FileChooserButton()
        grid.attach(dlg.fC, 1, 0, 1, 1)

        dlg.att_title = Gtk.Entry()
        grid.attach(dlg.att_title, 1, 1, 1, 1)

        dlg.att_desc = Gtk.Entry()
        grid.attach(dlg.att_desc, 1, 2, 1, 1)

        dlg.show_all()
        return dlg

    def upload_test_gui(self, *args):
        """Uploads test and attachments."""
        self.upload_test()

    def upload_test(self):
        """Uploads tests and attachments."""
        model = self.tree.get_model()
        lv_iter = model.get_iter_first()
        ngood = 0
        nbad = 0
        while lv_iter:
            past_iter = None
            values = model[lv_iter]
            payload = values[TestList.Json]
            payload["comments"] = values[TestList.Comments]
            payload["defects"] = values[TestList.Defects]

            rc = ITkDButils.upload_test(self.session, payload, values[TestList.Attachments], check_runNumber=True)
            if rc:
                ipos = rc.find("The following details may help:")
                if ipos>=0:
                    msg = rc[ipos:]
                else:
                    msg = rc
                dbGtkUtils.complain("Failed uploading test {}-{}\n".format(payload["component"], payload["testType"]), msg)
                self.write_message("Failed uploading test {}-{}\n{}\n".format(payload["component"], payload["testType"], msg))
                nbad += 1

            else:
                self.write_message("Upload {}-{} successfull\n".format(payload["component"], payload["testType"]))
                past_iter = lv_iter
                ngood += 1

            lv_iter = model.iter_next(lv_iter)
            if past_iter:
                model.remove(past_iter)

        if nbad>0:
            dbGtkUtils.complain("Failed to upload some tests", "{}/{} tests had errors.\nThey are left in the ListView.".format(nbad, ngood))
        else:
            dbGtkUtils.complain("All {} tests uploaded succesfully".format(ngood))


def main():
    """Main entry."""
    HELP_LINK="https://itkdb-gtk.docs.cern.ch/uploadMultipleTests.html"
    # DB login
    dlg = ITkDBlogin.ITkDBlogin()
    client = dlg.get_client()
    if client is None:
        print("Could not connect to DB with provided credentials.")
        dlg.die()
        sys.exit()

    client.user_gui = dlg

    # Start GUI
    UpT = UploadMultipleTests(client, help_link=HELP_LINK)

    if gtk_runs:
        UpT.present()
        UpT.connect("destroy", Gtk.main_quit)
        try:
            Gtk.main()

        except KeyboardInterrupt:
            print("Arrrgggg!!!")

    else:
        # Think
        pass

    dlg.die()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""send shipments to a selected site.

Items can be added via a QR reader.
"""
import pathlib
import sys
import re

try:
    import itkdb_gtk

except ImportError:
    from pathlib import Path
    cwd = Path(__file__).parent.parent
    sys.path.append(cwd.as_posix())

from itkdb_gtk import dbGtkUtils, ITkDBlogin, ITkDButils, QRScanner

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio

# Check if Gtk can be open
gtk_runs, gtk_args = Gtk.init_check()


class CreateShipments(dbGtkUtils.ITkDBWindow):
    """Create a shipment from input."""

    def __init__(self, session, help_link=None):
        """Initialization.

        Args:
            session: ITkDB session.

        """
        self.model = None
        self.tree = None
        self.shipments = {}
        self.attachment = None
        global gtk_runs
        if gtk_runs:
            super().__init__(session=session, title="Create Shipment",
                             help_link=help_link, gtk_runs=gtk_runs)
            self.init_window()

    def init_window(self):
        """Initialize window."""
        # set border width
        self.set_border_width(10)

        # intercept keyboard
        self.scanner = QRScanner.QRScanner(self.get_qrcode)

        # Prepare HeaderBar
        self.hb.props.title = "Create Shipment"

        # action button in header
        button = Gtk.Button()
        icon = Gio.ThemedIcon(name="document-send-symbolic")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        button.add(image)
        button.set_tooltip_text("Click to send shipment items.")
        button.connect("clicked", self.send_items)
        self.hb.pack_end(button)

        # Data panel
        grid = Gtk.Grid(column_spacing=5, row_spacing=1)
        self.mainBox.pack_start(grid, False, True, 0)

        # The recipient combo
        recipient = self.create_institute_combo()
        recipient.connect("changed", self.new_recipient)
        recipient.set_tooltip_text("Select the destination of the items.")
        lbl = Gtk.Label(label="Destination")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, 0, 1, 1)
        grid.attach(recipient, 1, 0, 1, 1)

        # Shipment name
        self.name = Gtk.Entry()
        self.name.set_tooltip_text("Set the name of the shipment")
        lbl = Gtk.Label(label="Name")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, 1, 1, 1)
        grid.attach(self.name, 1, 1, 1, 1)

        # Shipment comments.
        self.comments = Gtk.Entry()
        self.comments.set_tooltip_text("Type here your comments for hte shipment.")
        lbl = Gtk.Label(label="Comments")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, 2, 1, 1)
        grid.attach(self.comments, 1, 2, 1, 1)

        # The "Add/Remove/Send Item" buttons.
        box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        self.mainBox.pack_start(box, False, False, 0)

        dbGtkUtils.add_button_to_container(box, "Add Items",
                                           "Click to add a new items.\nAdd SNs separated by\n\tnew line,\n\ttab\n\tespace or\n\tcomma.",
                                           self.add_item)

        dbGtkUtils.add_button_to_container(box, "Remove Item",
                                           "Click to remove selected Item.",
                                           self.remove_item)

        dbGtkUtils.add_button_to_container(box, "Send Items",
                                           "Click to send items.",
                                           self.send_items)

        dbGtkUtils.add_button_to_container(box, "Add Attch.",
                                           "Click to add an attachment.",
                                           self.add_attachment)

        dbGtkUtils.add_button_to_container(box, "Remove Attch.",
                                           "Click to remove the attachment.",
                                           self.remove_attachment)

        # Paned object
        paned = Gtk.Paned(orientation=Gtk.Orientation.VERTICAL)
        paned.set_size_request(-1, 200)
        self.mainBox.pack_start(paned, True, True, 0)

        # The list of items
        tree_view = self.create_tree_view()
        paned.add1(tree_view)

        # The text view
        paned.add2(self.message_panel.frame)

        self.show_all()

    def create_tree_view(self, size=150):
        """Creates tree view with the list of items."""
        model = self.get_tree_view_model()
        self.tree = Gtk.TreeView(model=model)
        scrolled = Gtk.ScrolledWindow()
        scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        scrolled.add(self.tree)
        scrolled.set_size_request(-1, size)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Serial No.", renderer, text=0)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Nick", renderer, text=1)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Object Type", renderer, text=2)
        self.tree.append_column(column)

        renderer = Gtk.CellRendererText()
        column = Gtk.TreeViewColumn("Location", renderer, text=3)
        self.tree.append_column(column)

        return scrolled

    def new_recipient(self, combo):
        """A new recipient has been chosen."""
        name = self.get_institute_from_combo(combo)
        if name:
            self.recipient = name

    def new_destination(self, combo):
        """A new destination has been chosen."""
        name = self.get_institute_from_combo(combo)
        if name:
            self.destination = name

    def get_tree_view_model(self):
        """Return an empty model for the tree view."""
        return Gtk.ListStore(str, str, str, str, str)

    def add_item(self, *args):
        """Add new item."""
        # We will get the input either from a Dialog or from the QR reader.
        # For the later we get the SN in the argument list.
        if isinstance(args[0], Gtk.Button):
            txt = dbGtkUtils.get_a_value("Enter item SN", is_tv=True)
            if txt is None:
                return

            tmp = re.split(';|,| |\n|\t', txt)
            SNlist = [s.strip() for s in tmp if len(s.strip())>0]
        else:
            SNlist = [args[0]]

        for SN in SNlist:
            try:
                # Check that the object exists and get some information about it.
                rc = ITkDButils.get_DB_component(self.session, SN)
                if 'inTransit' in rc and rc['inTransit']:
                    dbGtkUtils.complain("Item {} is already in transit".format(SN),
                                        "This item is already in transit to {}".format(rc['shipmentDestination']['code']))
                    return

                nick = rc['alternativeIdentifier']
                obj_id = rc['id']
                obj = rc['componentType']['name']
                loc = rc['currentLocation']['code']
                serialN = rc['serialNumber']
                if serialN is None:
                    serialN = obj_id

                # Check tha tthe input is not already there
                model = self.tree.get_model()
                lv_iter = model.get_iter_first()
                while lv_iter:
                    if model.get_value(lv_iter, 0) == SN:
                        dbGtkUtils.complain("Duplicated item.",
                                            "Object {} is already in the list".format(SN))
                        return

                    lv_iter = model.iter_next(lv_iter)

                # Add the item in the liststore.
                model.append([serialN, nick, obj, loc, obj_id])

            except Exception:
                dbGtkUtils.complain("Error querying DB",
                                    "object {} does not exist.".format(SN))
                print(ITkDButils.get_db_response())

    def remove_item(self, *args):
        """Remove selected item."""
        select = self.tree.get_selection()
        model, lv_iter = select.get_selected()
        if lv_iter:
            values = model[lv_iter]
            rc = dbGtkUtils.ask_for_confirmation("Remove this items ?", values[0])
            if rc:
                model.remove(lv_iter)

    def add_attachment_dialog(self):
        """Create the add attachment dialog."""
        dlg = Gtk.Dialog(title="Add Attachment")
        dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                        Gtk.STOCK_OK, Gtk.ResponseType.OK)
        grid = Gtk.Grid(column_spacing=5, row_spacing=1)
        box = dlg.get_content_area()
        box.add(grid)

        lbl = Gtk.Label(label="File")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, 0, 1, 1)

        lbl = Gtk.Label(label="Title")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, 1, 1, 1)

        lbl = Gtk.Label(label="Description")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, 2, 1, 1)

        dlg.fC = Gtk.FileChooserButton()
        grid.attach(dlg.fC, 1, 0, 1, 1)

        dlg.att_title = Gtk.Entry()
        grid.attach(dlg.att_title, 1, 1, 1, 1)

        dlg.att_desc = Gtk.Entry()
        grid.attach(dlg.att_desc, 1, 2, 1, 1)

        dlg.show_all()
        return dlg

    def add_attachment(self, *args):
        """Add Attachment button clicked."""
        dlg = self.add_attachment_dialog()
        rc = dlg.run()
        if rc == Gtk.ResponseType.OK:
            try:
                path = pathlib.Path(dlg.fC.get_filename()).expanduser().resolve()
            except Exception as E:
                dbGtkUtils.complain("Could not create Attachment", str(E))
                dlg.hide()
                dlg.destroy()
                return

            T = dlg.att_title.get_text().strip()
            D = dlg.att_desc.get_text().strip()

            T = T if len(T) else None
            D = D if len(D) else None
            att = ITkDButils.Attachment(path=path, title=T, desc=D)
            self.attachment = att

        dlg.hide()
        dlg.destroy()

    def remove_attachment(self, *args):
        """Remove an existing attachment."""
        if self.attachment:
            self.attachment = None
            self.write_message("Attachment removed\n")
        else:
            self.write_message("No attachment found\n")

    def get_qrcode(self, txt):
        """Gets data from QR scanner."""
        self.write_message("{}\n".format(txt))

        # Try to add item to the list
        self.add_item(txt)

        return True

    def send_items(self, *args):
        """Send items in liststore."""
        model = self.tree.get_model()
        lv_iter = model.get_iter_first()
        items = []
        senders = {}
        while lv_iter:
            values = model[lv_iter]
            items.append(values[0])
            senders[values[3]] = senders.setdefault(values[3], 0) + 1
            lv_iter = model.iter_next(lv_iter)

        if len(items)>0:
            if len(senders) != 1:
                dbGtkUtils.complain("Too many senders.",
                                    "There are objects located in differen sites:{}".format('\n'.join(senders.keys())))
                return

            name = self.name.get_text()
            comments = self.comments.get_text()
            rc = ITkDButils.create_shipment(self.session,
                                            list(senders.keys())[0],
                                            self.recipient,
                                            items, name, True,
                                            comments=comments,
                                            attachment=self.attachment)
            if rc is None:
                rc = ITkDButils.get_db_response()
                ipos = rc.find("The following details may help:")
                msg = rc[ipos:]
                dbGtkUtils.complain("Error creting shipment.",
                                    "Could not send\n+ {}\n\n{}".format('\n+ '.join(items), msg))

            else:
                self.write_message("Successfully sent:\n+ {}\n".format('\n+ '.join(items)))
                model = self.get_tree_view_model()
                self.tree.set_model(model)
                self.comments.set_text("")
                self.name.set_text("")
                self.attachment = None

        else:
            self.write_message("Empty list of items when creating shipment.")


def main():
    """Main entry."""
    # main entry of the program
    HELP_LINK="https://itkdb-gtk.docs.cern.ch"

    dlg = ITkDBlogin.ITkDBlogin()
    client = dlg.get_client()
    if client is None:
        print("Could not connect to DB with provided credentials.")
        dlg.die()
        sys.exit()

    client.user_gui = dlg
    IS = CreateShipments(client, help_link="{}/createShipment.html".format(HELP_LINK))
    IS.set_accept_focus(True)
    IS.present()
    IS.connect("destroy", Gtk.main_quit)

    try:
        Gtk.main()

    except KeyboardInterrupt:
        print("Arrrgggg!!!")

    dlg.die()


if __name__ == "__main__":
    main()



"""A set of GTK utilities for DB scripts."""
import sys
import json
import time
import pathlib

from collections.abc import Iterable
from copy import deepcopy
from datetime import datetime
import webbrowser

import dateutil.parser
import numpy as np


import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, GObject, Gio, GLib


try:
    import itkdb_gtk

except ImportError:
    cwd = pathlib.Path(__file__).parent.parent
    sys.path.append(cwd.as_posix())

from itkdb_gtk import ITkDButils


def parse_date(txt):
    """Parse a date."""
    try:
        return dateutil.parser.parse(txt, fuzzy=False)

    except dateutil.parser.ParserError:
        return None


def parse_date_as_string(txt):
    """Parse data and return DB compatible string."""
    D = parse_date(txt)
    if D is None:
        return D

    out = D.isoformat(timespec='milliseconds')
    if out[-1] not in ['zZ']:
        out += 'Z'

    return out

def is_a_date(txt):
    """check tha the input string is a date."""
    try:
        tl = txt.lower()
        if len(txt)<5 and (tl!="now" and tl!="today"):
            return False

        dateutil.parser.parse(txt, fuzzy=False)
        return True

    except (dateutil.parser.ParserError, OverflowError):
        return False

def new_small_text_entry():
    """Returs a new, smaller Gtk.Entry."""
    entry = Gtk.Entry()
    provider = Gtk.CssProvider()
    style_context = entry.get_style_context()
    font_size = 2.25*style_context.get_property("font-size", 0)
    css = "entry {{ min-height: {}px; }}".format(font_size)
    provider.load_from_data(css.encode())
    style_context.add_provider(provider, Gtk.STYLE_PROVIDER_PRIORITY_SETTINGS)
    return entry

def set_entry_style(container):
    """Set max entry."""
    provider = Gtk.CssProvider()
    style_context = container.get_style_context()
    font_size = 2.25*style_context.get_property("font-size", 0)
    css = "{} {{ min-height: {}px; }}".format(container.get_name(), font_size)
    provider.load_from_data(css.encode())
    style_context.add_provider(provider, Gtk.STYLE_PROVIDER_PRIORITY_SETTINGS)
    return container

def set_button_color(btn, bg_color, fg_color="white"):
    """Set button color"""
    css = "#{} {{background-image: none; background-color: {}; color: {}}}".format(btn.get_name(), bg_color, fg_color)
    provider = Gtk.CssProvider()
    provider.load_from_data(css.encode())
    style_context = btn.get_style_context()
    style_context.add_provider(provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)

def set_combo_iter(combo, txt, col=0):
    """Set scombo active iter to that containing txt in column col."""
    model = combo.get_model()
    lv_iter = model.get_iter_first()
    while lv_iter:
        val = model.get_value(lv_iter, col)
        if val == txt:
            combo.set_active_iter(lv_iter)
            break

        lv_iter = model.iter_next(lv_iter)


def is_iterable(obj):
    """Tell if an object is iterable. Strings are not considered iterables."""
    if isinstance(obj, Iterable):
        if isinstance(obj, str) or isinstance(obj, bytes):
            return False
        else:
            return True
    else:
        return False


class MyEncoder(json.JSONEncoder):
    """To truncate the number of decimals in floats."""

    def default(self, o):
        """Format floats or deletate to JSonEncoder."""
        if isinstance(o, datetime):
            text = o.astimezone().isoformat()
            return text
        elif isinstance(o, np.integer):
            return int(o)
        elif isinstance(o, np.floating):
            return float(o)
        elif isinstance(o, np.ndarray):
            return o.tolist()
        else:
            return super().default(o)


def empty_container(container):
    """Remove all children from a container.

    Args:
        container: The container.

    """
    chilren = [C for C in container.get_children()]
    for C in chilren:
        container.remove(C)


def replace_in_container(container, child):
    """Replace a child from a single-child container.

    Args:
        container: the container
        child: the new child to be added

    """
    empty_container(container)
    if isinstance(container, Gtk.Box):
        container.pack_end(child, True, True, 0)
    else:
        container.add(child)
    container.show_all()


def complain(main_title, second_text="", parent=None):
    """Open an error dialog.

    Args:
        main_title: Main text in window
        second_text: Second text
        parent: dialog parent

    """
    dialog = Gtk.MessageDialog(
        transient_for=parent,
        flags=0,
        message_type=Gtk.MessageType.ERROR,
        buttons=Gtk.ButtonsType.OK,
        text=main_title,
    )
    dialog.format_secondary_text(second_text)
    dialog.run()
    dialog.destroy()


def ask_for_confirmation(main_title, second_text, parent=None):
    """Ask for action cofirmation.

    Args:
        main_title: Main title in the message window
        second_text: Secondary text in the message widow
        parent (optional): The parent window. Defaults to None.

    Return:
        OK: True if OK button clicked.

    """
    dialog = Gtk.MessageDialog(
        transient_for=parent,
        flags=0,
        message_type=Gtk.MessageType.INFO,
        text=main_title
    )
    dialog.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OK, Gtk.ResponseType.OK)
    dialog.format_secondary_text(second_text)
    out = dialog.run()
    dialog.destroy()
    return (out == Gtk.ResponseType.OK)

class TextEntry(GObject.GObject):
    """Create a Gtk text entry/view object."""
    __gsignals__ = {
        "text_changed": (GObject.SIGNAL_RUN_FIRST, None, (str,))
    }

    def __init__(self, n_lines=1, small=False):
        """Init."""
        GObject.GObject.__init__(self)
        self.tmp_txt = ""
        self.nlines = n_lines
        self.do_emit = True
        if self.nlines > 1:
            self.widget = Gtk.Frame()
            scrolled = Gtk.ScrolledWindow()
            scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
            scrolled.set_hexpand(True)
            scrolled.set_vexpand(True)

            self.widget.add(scrolled)
            self.entry = Gtk.TextView()
            scrolled.add(self.entry)

        else:
            if small:
                self.widget = new_small_text_entry()
            else:
                self.widget = Gtk.Entry()

            self.widget.connect("focus-in-event", self.on_enter)
            self.widget.connect("focus-out-event", self.on_leave)
            self.entry = self.widget

    def do_my_signal(self, *args):
        """Signal handler."""
        return

    def on_enter(self, *args):
        """On enter."""
        self.tmp_txt = self.widget.get_text().strip()
        return False

    def on_leave(self, *args):
        """On leave."""
        val = self.widget.get_text().strip()
        if val != self.tmp_txt:
            self.do_emit = False
            self.emit("text_changed", val)
            self.do_emit = True

    def get_text(self):
        """Return the text."""
        if self.nlines > 1:
            buff = self.entry.get_buffer()
            start = buff.get_start_iter()
            end = buff.get_end_iter()
            return buff.get_text(start, end, False)

        else:
            return self.entry.get_text()

    def set_text(self, text):
        """Sets text."""
        if text is None:
            return

        if self.nlines > 1:
            self.entry.get_buffer().set_text(text)
        else:
            self.entry.set_text(text)
            if self.do_emit:
                self.do_emit = False
                self.emit("text_changed", text)
                self.do_emit = True

def get_a_value(main_title, second_text=None, is_tv=False, parent=None):
    """Open a dialog to get a value.

    Args:
        main_title: Main title in the message window
        is_tv: If true show a text view rathar than an entry.
        second_text: Secondary text in the message widow
        parent (optional): The parent window. Defaults to None.

    Return:
        value: The value in the entry

    """
    dlg = Gtk.Dialog(title="Get a Value",
                     transient_for=parent,
                     flags=0)
    dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                    Gtk.STOCK_OK, Gtk.ResponseType.OK)
    area = dlg.get_content_area()
    box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
    area.add(box)

    box.pack_start(Gtk.Label(label=main_title), False, True, 0)
    if second_text and len(second_text):
        box.pack_start(Gtk.Label(label=second_text), False, True, 0)

    entry = TextEntry(3 if is_tv else -1)

    box.pack_start(entry.widget, False, True, 0)
    dlg.show_all()

    rc = dlg.run()
    if rc == Gtk.ResponseType.OK:
        out = entry.get_text().rstrip()
    else:
        out = None

    dlg.hide()
    dlg.destroy()

    return out


def get_a_list_of_values(main_title, labels, defaults=None, second_text=None, parent=None):
    """Get a list of values.

    Args:
        main_title: Main title for window
        labels: List of labes to get the values. If the label ends with /v
                then a TextView will be shown instead of a TextEntry.
        defaults (optional): default values-
        second_text (optional): Second title for window-. Defaults to None.

    Returns:
        list with values

    """
    dlg = Gtk.Dialog(title="Get List of values",
                     transient_for=parent,
                     flags=0)

    dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                    Gtk.STOCK_OK, Gtk.ResponseType.OK)
    area = dlg.get_content_area()
    box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
    area.pack_start(box, False, True, 2)

    lbl = Gtk.Label()
    lbl.set_markup("<b>{}</b>".format(main_title))
    box.pack_start(lbl, False, True, 5)
    if second_text and len(second_text):
        box.pack_start(Gtk.Label(label=second_text), False, True, 0)

    entries = []
    values = []
    is_text_view = []
    vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
    area.pack_start(vbox, False, True, 0)
    for i, txt in enumerate(labels):
        use_tv = False
        if txt.endswith("/v"):
            is_text_view.append(True)
            use_tv = True
            txt = txt[:-2]
        else:
            is_text_view.append(False)

        lbl = Gtk.Label(label=txt)
        lbl.set_justify(Gtk.Justification.LEFT)
        vbox.pack_start(lbl, False, False, 0)

        entry = TextEntry(3 if use_tv else -1)
        try:
            entry.set_text(defaults[i])

        except (TypeError, IndexError):
            pass

        vbox.pack_start(entry.widget, False, False, 0)
        entries.append(entry)

    dlg.show_all()
    rc = dlg.run()
    if rc == Gtk.ResponseType.OK:
        for entry, is_tv in zip(entries, is_text_view):
            values.append(entry.get_text())

    dlg.hide()
    dlg.destroy()
    return values


def add_button_to_container(box, label, tooltip=None, callback=None):
    """Creates a buttons and adds to container.

    Args:
        box: The container.
        label: The button label
        tooltip (optional): Tooltip message. Defaults to None.
        callback (optional): callback function. Defaults to None.

    """
    btn = Gtk.Button(label=label)

    if tooltip:
        btn.set_tooltip_text(tooltip)

    if callback:
        btn.connect("clicked", callback)

    box.pack_start(btn, True, False, 0)

    return btn


class MessagePanel(object):
    """Encapsulates a TExtView object to show messages."""

    def __init__(self, size=100):
        """Initializarion."""
        self.frame = None
        self.text_view = Gtk.TextView()
        self.textbuffer = self.text_view.get_buffer()

        self.__create_message_panel(size)


    def __create_message_panel(self, size):
        """Creates a message panel within a frame.

        Args:
            size: size of the panel

        Returns:
            Gtk.TextBuffer, Gtk.Frame
        """
        frame = Gtk.Frame()
        frame.set_shadow_type(Gtk.ShadowType.IN)

        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        box.set_size_request(-1, size)
        frame.add(box)

        # The title for the tet view
        box.pack_start(Gtk.Label(label="Messages"), False, True, 0)

        # A scroll window with the text view
        scrolled = Gtk.ScrolledWindow()
        scrolled.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
        scrolled.add(self.text_view)
        box.pack_start(scrolled, True, True, 0)
        self.frame = frame

    def scroll_to_end(self):
        """Scrolls text view to end."""
        end = self.textbuffer.get_end_iter()
        self.text_view.scroll_to_iter(end, 0, False, 0, 0)

    def write_message(self, text, write_date=True):
        """Writes text to Text Viewer."""
        nlines = self.textbuffer.get_line_count()
        if nlines > 100:
            start = self.textbuffer.get_iter_at_line(0)
            end = self.textbuffer.get_iter_at_line(75)
            self.textbuffer.delete(start, end)

        end = self.textbuffer.get_end_iter()
        if write_date:
            msg = "[{}]  {}".format(time.strftime("%d/%m/%y %T"), text)
        else:
            msg = text

        self.textbuffer.insert(end, msg)
        GLib.idle_add(self.scroll_to_end)

    def write(self, txt):
        """A write method."""
        self.write_message(txt, write_date=False)


class ITkDBWindow(Gtk.Window):
    """Base class for GUI main windows."""

    def __init__(self, title="", session=None, show_search=None, help_link=None, gtk_runs=True, panel_size=100):
        """Initialization.

        Args:
            title: The title of the window.
            session: ITkDB session.
            show_search: tooltip for search button in header (calls to query_db).
                         No search button if this is None.
            gtk_runs: If False, Gtk could not be loaded and app should be
                      terminal only.

        """
        self.session = session
        self.inst2code = {}
        self.code2inst = {}
        self.message_panel = None
        self.help = help_link
        self.pdb_user = ITkDButils.get_db_user(self.session)

        if gtk_runs:
            super().__init__(title=title)
            self.prepare_window(title, show_search, panel_size)

    def prepare_window(self, title, show_search, panel_size):
        """Inititalizes GUI."""
        # Prepare HeaderBar
        self.hb = Gtk.HeaderBar()
        self.hb.set_show_close_button(True)
        self.set_titlebar(self.hb)

        self.userLabel = Gtk.Button.new_with_label(self.session.user.name)
        self.userLabel.set_tooltip_text("Click to change user.")
        if hasattr(self.session, "user_gui"):
            self.session.user_gui.connect("new_login", self.new_login)
        self.userLabel.connect("clicked", self.reconnect)
        self.hb.pack_start(self.userLabel)

        if show_search:
            button = Gtk.Button()
            icon = Gio.ThemedIcon(name="system-search-symbolic")
            image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
            button.add(image)
            button.set_tooltip_text(show_search)
            button.connect("clicked", self.query_db)
            self.hb.pack_end(button)

        if self.help:
            button = Gtk.Button()
            icon = Gio.ThemedIcon(name="help-browser-symbolic")
            image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
            button.add(image)
            button.connect("clicked", self.show_help)
            self.hb.pack_end(button)


        # Create main content box
        self.mainBox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        self.mainBox.set_property("margin-left", 6)
        self.mainBox.set_property("margin-right", 6)

        self.title_label = None
        if len(title)>0:
            lbl = Gtk.Label()
            lbl.set_markup("<big><b>{}\n</b></big>".format(title))
            lbl.set_xalign(0.5)
            self.mainBox.pack_start(lbl, False, False, 2)
            self.title_label = lbl

        self.add(self.mainBox)

        # The text view and buffer
        self.message_panel = MessagePanel(size=panel_size)

        # The button box
        btnBox = Gtk.ButtonBox(orientation=Gtk.Orientation.HORIZONTAL)

        btn = Gtk.Button(label="Quit")
        btn.connect("clicked", self.quit)
        btnBox.add(btn)

        self.mainBox.pack_end(btnBox, False, True, 0)

    def set_window_title(self, title):
        """Set window title."""
        hb = self.get_titlebar()
        hb.props.title = title
        if self.title_label:
            self.title_label.set_markup("<big><b>{}\n</b></big>".format(title))

    def quit(self, *args):
        """Quits the application."""
        self.hide()
        self.destroy()

    def show_help(self, *args):
        """Show help"""
        webbrowser.open(self.help)

    def query_db(self, *args):
        """Search button clicked."""
        return

    def new_login(self, obj, msg):
        """A new user logged in."""
        if msg == "<OK>":
            if hasattr(self.session, "user_gui"):
                self.session = self.session.user_gui.get_client()

            if self.userLabel.get_child():
                self.userLabel.get_child().set_text(self.session.user.name)

        else:
            self.write_message("Could not login.\n{}".format(msg))

    def reconnect(self, *args):
        """Reconnects."""
        if hasattr(self.session, "user_gui"):
            self.session.user_gui.reconnect(force=True)

    def create_institute_combo(self, only_user=False):
        """Create a combe with all institutes.
        
        Args:
            only_user: if True, add only institutes the user belongs to.
            
        """
        compltn = self.get_institute_list(only_user)
        combo = Gtk.ComboBox.new_with_model_and_entry(compltn.get_model())
        combo.set_entry_text_column(0)
        combo.get_child().set_completion(compltn)

        return combo

    def get_institute_list(self, only_user=False):
        """Get the institute list.
        
        Args:
            only_user: if True, add only institutes the user belongs to.
        
        """
        if only_user and self.pdb_user:
            sites = self.pdb_user["institutions"]
        else:
            sites = self.session.get("listInstitutions", json={})
            
        liststore = Gtk.ListStore(str, str)
        for site in sites:
            self.code2inst[site['code']] = site['name']
            self.inst2code[site['name']] = site['code']
            liststore.append([site["code"], site["code"]])
            liststore.append([site["name"], site["code"]])

        completion = Gtk.EntryCompletion()
        completion.set_model(liststore)
        completion.set_text_column(0)
        return completion

    def get_institute_from_combo(self, combo):
        """Get Institute from combo."""
        tree_iter = combo.get_active_iter()
        if tree_iter is not None:
            model = combo.get_model()
            name = model[tree_iter][1]

        else:
            name = combo.get_child().get_text()
            if name in self.inst2code:
                name = self.inst2code[name]

            elif name not in self.code2inst:
                name = None

        return name

    def create_text_view(self, size=50):
        """Create a frame wit hteh text view."""
        frame = Gtk.Frame()
        frame.set_shadow_type(Gtk.ShadowType.IN)

        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        box.set_size_request(-1, size)
        frame.add(box)

        # The title for the tet view
        box.pack_start(Gtk.Label(label="Messages"), False, True, 0)

        # A scroll window with the text view
        scrolled = Gtk.ScrolledWindow()
        scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        scrolled.add(self.text_view)
        box.pack_start(scrolled, True, True, 0)

        return frame

    def write_message(self, text):
        """Writes text to Text Viewer."""
        self.message_panel.write_message(text)


class DictDialog(Gtk.Grid):
    """Creates a dialog to show and edit variables in a JSon dict."""

    def __init__(self, values, hidden_keys=None):
        """Create the Gtk.Grid.

        Args:
            values: A dict (JSon-like)
            hidden_keys: keys tha twill not be shown.

        """
        super().__init__(column_spacing=5, row_spacing=1)

        self.set_border_width(10)
        self.factory = deepcopy(values)
        self.values = deepcopy(values)
        self.keys = {}
        self.containers = {}
        self.hidden_keys = hidden_keys if hidden_keys else {}
        self.show_values()

    def factory_reset(self):
        """Set values to original values."""
        self.values = deepcopy(self.factory)
        self.refresh()

    def on_enter(self, entry, *args):
        """Get the value when we first enter into the Entry."""
        self.keys[args[2]] = entry.get_text()

    def on_leave(self, entry, event, value, name):
        """Check, when leaving the entry, if the value has changed."""
        def str2bool(v):
            return v.lower() in ("yes", "true", "t", "1")

        val = self.keys[name]
        txt = entry.get_text()
        if val != txt:
            keys = name.split('.')
            itm = self.values
            for k in keys[:-1]:
                itm = itm[k]

            last_key = keys[-1]
            if isinstance(itm, tuple) or isinstance(itm, list):
                indx = int(last_key)
                tp = type(itm[indx])
                itm[indx] = tp(txt)

            elif isinstance(itm[last_key], bool):
                itm[last_key] = str2bool(txt)

            elif isinstance(itm[last_key], datetime):
                itm[last_key] = dateutil.parser.parse(txt)

            elif is_a_date(itm[last_key]):
                D = dateutil.parser.parse(txt)
                out = D.isoformat(timespec='milliseconds')
                if out[-1] not in ['zZ']:
                    out += 'Z'

                itm[last_key] = out
                self.containers[name].set_text(out)

            else:
                if itm[last_key] is None:
                    itm[last_key] = txt
                else:
                    tp = type(itm[last_key])
                    itm[last_key] = tp(txt)

    def show_item(self, value, name=None):
        """Handle a single item."""
        if isinstance(value, list) or isinstance(value, tuple):
            container = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
            for i, v in enumerate(value):
                key_name = "{}.{}".format(name, i)
                container.pack_start(self.show_item(v, key_name), True, True, 0)

        elif isinstance(value, dict):
            container = Gtk.Grid(column_spacing=5, row_spacing=0)
            irow = 0
            for key, val in value.items():
                if name:
                    key_name = "{}.{}".format(name, key)
                lbl = Gtk.Label(label=key)
                lbl.set_xalign(0.0)
                if is_iterable(val):
                    lbl.set_yalign(0.0)
                else:
                    lbl.set_yalign(0.5)

                container.attach(lbl, 0, irow, 1, 1)
                container.attach(self.show_item(val, key_name), 1, irow, 1, 1)
                irow = irow + 1

        else:
            if value is None:
                value = ""

            container = Gtk.Entry(text=value)
            provider = Gtk.CssProvider()
            style_context = container.get_style_context()
            font_size = 2.25*style_context.get_property("font-size", 0)
            css = "entry {{ min-height: {}px; }}".format(font_size)
            provider.load_from_data(css.encode())
            style_context.add_provider(provider, Gtk.STYLE_PROVIDER_PRIORITY_SETTINGS)
            container.connect("populate-popup", self.add_insert_path)

            if name:
                container.set_name(name)
                self.containers[name] = container

            container.connect("focus-in-event", self.on_enter, value, name)
            container.connect("focus-out-event", self.on_leave, value, name)

        return container

    def add_insert_path(self, entry, menu, *args):
        """Adds a new item in the pop-up menu."""
        item = Gtk.MenuItem(label="Get file path")
        item.connect("activate", self.on_set_path, entry)
        menu.append(item)
        menu.show_all()

    def on_set_path(self, menu_item, entry):
        """Sets the path to the entry."""
        fdlg = Gtk.FileChooserNative(action=Gtk.FileChooserAction.OPEN, accept_label="Select")
        response = fdlg.run()
        if response == Gtk.ResponseType.ACCEPT:
            ifiles = [ipath for ipath in fdlg.get_filenames()]
            if len(ifiles)<1:
                return
            if len(ifiles) > 1:
                complain("More than one file selected","Choosing first.")

            fnam = ifiles[0]
            entry.set_text(fnam)
            self.on_leave(entry, None, None, entry.get_name())

    def set_value(self, key, value):
        """Set value of a container and key."""
        try:
            self.containers[key].set_text("{}".format(value))
            self.keys[key] = value
        except KeyError:
            complain("Key {} does not exist in DictDialog".format(key))

    def show_values(self):
        """Show the keys and values of a dictionary (JSON).

        Args:
            values: The dictionary object

        """
        irow = 0
        self.set_row_spacing(5)
        for key, value in self.values.items():
            if key in self.hidden_keys:
                continue

            lbl_key = Gtk.Label(label=key)
            lbl_key.set_xalign(0.0)
            if is_iterable(value):
                lbl_key.set_yalign(0.0)
            else:
                lbl_key.set_yalign(0.5)

            container = self.show_item(value, key)

            self.attach(lbl_key, 0, irow, 1, 1)
            self.attach(container, 1, irow, 1, 1)
            irow = irow + 1

    def refresh(self):
        """Refresh all values."""
        empty_container(self)
        self.show_values()
        self.show_all()
        self.queue_draw()

    @staticmethod
    def create_json_data_editor(data):
        """Create a dialog to show the JSon file."""
        dlg = Gtk.Dialog(title="Test Data")
        dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                        Gtk.STOCK_OK, Gtk.ResponseType.OK)

        dlg.set_property("height-request", 500)
        box = dlg.get_content_area()
        value = DictDialog(data)
        scrolled = Gtk.ScrolledWindow()
        scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        scrolled.add(value)
        box.pack_start(scrolled, True, True, 10)

        dlg.show_all()
        rc = dlg.run()
        dlg.hide()
        dlg.destroy()

        return value.values, rc


def create_scrolled_dictdialog(the_dict, hidden=("component", "testType")):
    """Create a DictDialog within a scrolled window.

    Args:
        the_dict: the input dictionary with values.

    Returns:
        scrolled: the scrolled window
        gM: the DictDialog

    """
    gM = DictDialog(the_dict, hidden)
    scrolled = Gtk.ScrolledWindow()
    scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
    scrolled.add(gM)
    return scrolled, gM


def main():
    """Main entry."""
    result = {
        "component": "the_serial_nukber",
        "testType": "METROLOGY_AVS",
        "institution": "...",
        "runNumber": "...",
        "date": datetime.now(),
        "passed": True,
        "problems": False,
        "properties": {
            "OPERATOR": "operator"
        },
        "results": {
            "LOCATOR1_DIAMETER": True,
            "LOCATOR1_X": -1,
            "LOCATOR1_Y": -1,
            "LOCATOR2_DIAMETER": True,
            "LOCATOR2_X": -1,
            "LOCATOR2_Y": -1,
            "LOCATOR3_DIAMETER": -1,
            "FIDUCIAL1_DIAMETER": -1,
            "FIDUCIAL1_X": -1,
            "FIDUCIAL1_Y": -1,
            "FIDUCIAL2_X": -1,
            "FIDUCIAL2_Y": -1,
            "FIDUCIAL2_DIAMETER": -1,
            "COPLANARITY_FRONT": -1,
            "PARALLELISM_FRONT": -1,
            "COPLANARITY_BACK": -1,
            "PARALLELISM_BACK": -1,
            "ANGLE_VCHANNEL": -1,
            "ENVELOPE": -1,
            "FLAT_LOCAL_BACK": [
                543.0948648254819,
                543.0948648254819
            ]
        }
    }

    def on_button_clicked(widget, *args):
        """Dump the JSON thing."""
        print(len(args))
        print(json.dumps(args[0].values, indent=3, cls=MyEncoder))

    win = Gtk.Window()
    win.connect("destroy", Gtk.main_quit)
    box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
    win.add(box)

    value = DictDialog(result)
    win.set_property("height-request", 500)
    scrolled = Gtk.ScrolledWindow()
    scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
    scrolled.add(value)
    box.pack_start(scrolled, True, True, 10)

    button = Gtk.Button(label="Click Here")
    button.connect("clicked", on_button_clicked, value)
    box.pack_start(button, False, False, 0)

    win.show_all()
    Gtk.main()

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""GUI to upload tests."""
import argparse
import json
import sys
from pathlib import Path

try:
    import itkdb_gtk

except ImportError:
    cwd = Path(__file__).parent.parent
    sys.path.append(cwd.as_posix())

from itkdb_gtk import dbGtkUtils, ITkDBlogin, ITkDButils
from itkdb_gtk.ShowComments import ShowComments
from itkdb_gtk.ShowAttachments import ShowAttachments
from itkdb_gtk.ShowDefects import ShowDefects

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gio

# Check if Gtk can be open
gtk_runs, gtk_args = Gtk.init_check()


def create_json_data_editor(data):
    """Create a dialog to show the JSon file."""
    dlg = Gtk.Dialog(title="Test Data")
    dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                    Gtk.STOCK_OK, Gtk.ResponseType.OK)

    dlg.set_property("height-request", 500)
    box = dlg.get_content_area()
    value = dbGtkUtils.DictDialog(data)
    scrolled = Gtk.ScrolledWindow()
    scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
    scrolled.add(value)
    box.pack_start(scrolled, True, True, 10)

    dlg.show_all()
    return value, dlg


def check_data(data):
    """Checks validity of JSon data.

    Args:
        data (): The json data

    Returns:
        boolean: True if valid, False otherwise.

    """
    errors = []
    missing = []
    if "component" not in data:
        errors.append("Need reference to component, hex string")
        missing.append("component")

    if "testType" not in data:
        errors.append("Need to know test type, short code")
        missing.append("testType")

    if "institution" not in data:
        errors.append("Need to know institution, short code")
        missing.append("institution")

    if "results" not in data:
        errors.append("Need some test results")
        missing.append("results")

    return errors, missing


class UploadTest(dbGtkUtils.ITkDBWindow):
    """Collects information to upload a test and its attachments."""

    def __init__(self, session, payload=None, attachment=None, help_link=None):
        """Initialization.

        Args:
            session: ITkDB session
            payload: path of test file
            attachment: an Attachment object or list of attachments.

        """
        self.payload = payload
        self.data = None
        self.folder = None
        self.attachments = []
        self.comments = []
        self.defects = []
        self.currentStage = None
        if attachment is not None:
            if isinstance(attachment, ITkDButils.Attachment):
                if attachment.path is not None:
                    self.attachments.append(attachment)
            else:
                try:
                    for att in attachment:
                        self.attachments.append(att)

                except TypeError:
                    print("Wrong attachment: {}".format(attachment))

        global gtk_runs
        if gtk_runs:
            super().__init__(session=session, title="Upload Test", gtk_runs=gtk_runs, help_link=help_link)
            self.init_window()

    def init_window(self):
        """Creates the Gtk window."""
        # Initial tweaks
        self.set_border_width(10)

        # Prepare HeaderBar
        self.hb.props.title = "Upload Tests"

        # Active button in header
        button = Gtk.Button()
        icon = Gio.ThemedIcon(name="document-send-symbolic")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        button.add(image)
        button.set_tooltip_text("Click to upload test")
        button.connect("clicked", self.upload_test_gui)
        self.hb.pack_end(button)

        # Data panel
        grid = Gtk.Grid(column_spacing=5, row_spacing=1)
        self.mainBox.pack_start(grid, True, False, 0)

        # The test file widgets
        lbl = Gtk.Label(label="Test file")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, 0, 1, 1)

        self.testF = Gtk.FileChooserButton()
        self.testF.set_tooltip_text("Click to select JSon test file.")

        grid.attach(self.testF, 1, 0, 1, 1)
        self.testF.connect("file-set", self.on_test_file)

        # This is to show/edit the test file data
        btn = Gtk.Button()
        icon = Gio.ThemedIcon(name="view-paged-symbolic")
        image = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.BUTTON)
        btn.add(image)
        btn.set_tooltip_text("Click to view/edit test data.")
        btn.connect("clicked", self.show_data)
        grid.attach(btn, 2, 0, 1, 1)

        # Object Data
        lbl = Gtk.Label(label="Serial Number")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, 1, 1, 1)

        self.entrySN = Gtk.Entry()
        grid.attach(self.entrySN, 1, 1, 1, 1)

        # Test type
        lbl = Gtk.Label(label="Test Type")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, 2, 1, 1)

        self.entryTest = Gtk.Entry()
        grid.attach(self.entryTest, 1, 2, 1, 1)

        # Object Stage
        lbl = Gtk.Label(label="Object Stage")
        lbl.set_xalign(0)
        grid.attach(lbl, 0, 3, 1, 1)

        self.objStage = Gtk.ComboBoxText.new()
        self.objStage.connect("changed", self.on_new_stage)
        grid.attach(self.objStage, 1, 3, 1, 1)

        self.isRetroactive = Gtk.ToggleButton.new_with_label("RetroActive")
        grid.attach(self.isRetroactive, 2, 3, 1, 1)


        # The "Add attachment" button.
        box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        self.mainBox.pack_start(box, False, False, 0)
        self.btn_attch = dbGtkUtils.add_button_to_container(box, "Attachments",
                                           "Click to edit attachments.",
                                           self.edit_attachments)

        self.btn_comments = dbGtkUtils.add_button_to_container(box, "Comments",
                                           "Click to edit comments.",
                                           self.edit_comments)

        self.btn_defects = dbGtkUtils.add_button_to_container(box, "Defects",
                                           "Click to edit defects.",
                                           self.edit_defects)

        dbGtkUtils.add_button_to_container(box, "Upload Test",
                                           "Click to upload test.",
                                           self.upload_test_gui)

        # The text view
        self.mainBox.pack_start(self.message_panel.frame, True, True, 0)

        self.show_all()

        if self.payload:
            try:
                the_path = Path(self.payload).expanduser().resolve()
                if the_path.exists():
                    ifile = Path(self.payload).expanduser().resolve().as_posix()
                    self.testF.set_filename(ifile)
                    self.on_test_file(self.testF)
                    self.write_message("Loaded {}\n".format(the_path.name))

                else:
                    print("Input file does not exists: {}".format(self.payload))

            except TypeError:
                self.load_payload(self.payload)
                self.write_message("Loaded memory payload.\n")
                self.testF.set_sensitive(False)

        if len(self.attachments) > 0:
            self.btn_attch.set_label("Attachments ({})".format(len(self.attachments)))

        if len(self.comments) > 0:
            self.btn_comments.set_label("Comments ({})".format(len(self.comments)))

        if len(self.defects) > 0:
            self.btn_defects.set_label("Defects ({})".format(len(self.defects)))


    def get_test_institute(self):
        """Select an institue."""
        dlg = Gtk.Dialog(title="Select Institution.", flags=0)
        dlg.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                        Gtk.STOCK_OK, Gtk.ResponseType.OK)
        area = dlg.get_content_area()
        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        area.add(box)

        box.pack_start(Gtk.Label(label="Select an Institute"), False, True, 0)

        combo = self.create_institute_combo()
        box.pack_start(combo, False, True, 5)

        dlg.show_all()
        rc = dlg.run()

        out = None
        if rc == Gtk.ResponseType.OK:
            out = self.get_institute_from_combo(combo)

        dlg.hide()
        dlg.destroy()
        return out

    def load_payload(self, data):
        """Payload is given as a dict."""
        self.data = data
        errors, missing = check_data(self.data)
        self.complete_missing(missing, errors)
        self.set_stages()

    def on_new_stage(self, *args):
        """New stage selected."""
        stg = self.objStage.get_active_text()
        changed = (stg != self.currentStage)
        self.isRetroactive.set_active(changed)
        self.data["isRetroactive"] = changed
        if changed:
            self.data["stage"] = stg
        else:
            if "stage" in self.data:
                del self.data["stage"]

    def set_stages(self):
        """Prepare the stages combo."""
        # Check the object stage
        SN = self.data["component"]
        try:
            obj = ITkDButils.get_DB_component(self.session, SN)
            self.currentStage = obj["currentStage"]["code"]

            self.objStage.remove_all()
            indx = 0
            for i, stg in enumerate(obj["stages"]):
                S = stg["code"]
                self.objStage.append_text(S)
                if S == self.currentStage:
                    indx = i

            self.objStage.set_active(indx)

        except Exception:
            self.write_message("Something went wrong with the stages\n")


    def on_test_file(self, fdlg):
        """Test file browser clicked."""
        fnam = fdlg.get_filename()
        self.folder = Path(fnam).parent

        # The file exists by definition
        try:
            self.data = json.loads(open(fnam, encoding="UTF-8").read())
            errors, missing = check_data(self.data)
            self.complete_missing(missing, errors)
            self.set_stages()

        except Exception as E:
            self.data = None
            self.write_message("Cannot load file {}\n".format(fnam))
            self.write_message("{}\n".format(str(E)))

    def complete_missing(self, missing, errors):
        """Completes missing parts."""
        if len(missing):
            self.write_message("Some keys are missing in the JSon file.\n")
            self.write_message("{}\n".format("\n".join(['\t'+line for line in missing])))

            if "institution" in missing and len(missing) == 1:
                site = self.get_test_institute()
                if site:
                    self.data["institution"] = site
                    self.write_message("Setting Institution to {}\n".format(self.data["institution"]))

            else:
                dbGtkUtils.complain("Invalid JSON data\n{}".format('\n'.join(errors)))

        self.find_attachments()
        self.find_comments()
        self.find_defects()

        self.entrySN.set_text(self.data["component"] if self.data["component"] else "")
        self.entryTest.set_text(self.data["testType"])
        self.comments = self.data.get("comments", [])
        self.defects = self.data.get("defects", [])


    def show_data(self, *args):
        """Show data button clicked."""
        if self.data is None:
            return

        value, dlg = create_json_data_editor(self.data)
        rc = dlg.run()
        if rc == Gtk.ResponseType.OK:
            self.data = value.values

        dlg.hide()
        dlg.destroy()

    def find_attachments(self):
        """Find Attachments in payload."""
        # We used to clean the attachmetns, but this would remove the ones given
        # in the contructor.
        # self.attachments = []
        if "attachments" in self.data:
            for A in self.data["attachments"]:
                if not Path(A["path"]).exists():
                    if self.folder:
                        the_path = self.folder.joinpath(A["path"])
                    else:
                        continue
                else:
                    the_path = Path(A["path"]).expanduser().resolve()

                self.attachments.append(ITkDButils.Attachment(path=the_path, title=A["title"], desc=A["description"]))

        if len(self.attachments) > 0:
            self.btn_attch.set_label("Attachments ({})".format(len(self.attachments)))

    def edit_attachments(self, *args):
        """Edit test attachmetns."""
        SA = ShowAttachments("Test Attachments", self.session, self.attachments, parent=self)
        response = SA.run()
        if response == Gtk.ResponseType.OK:
            self.attachments = SA.attachments

        SA.hide()
        SA.destroy()

        if len(self.attachments) > 0:
            self.btn_attch.set_label("Attachments ({})".format(len(self.attachments)))

    def find_comments(self):
        """Find comments in payload"""
        self.comments = []
        if "comments" in self.data:
            for C in self.data["comments"]:
                self.comments.append(C)

        if len(self.comments) > 0:
            self.btn_comments.set_label("Comments ({})".format(len(self.comments)))

    def edit_comments(self, *args):
        """Edit test comments."""
        SC = ShowComments("Test Comments", self.comments, self)
        rc = SC.run()
        if rc == Gtk.ResponseType.OK:
            self.comments = SC.comments

        SC.hide()
        SC.destroy()

        if len(self.comments) > 0:
            self.btn_comments.set_label("Comments ({})".format(len(self.comments)))


    def find_defects(self):
        """Find defects in payload."""
        self.defects = []
        if "defects" in self.data:
            for D in self.data["defects"]:
                self.defects.append(D)

        if len(self.defects) > 0:
            self.btn_defects.set_label("Defects ({})".format(len(self.defects)))

    def edit_defects(self, *args):
        """Edit test defects."""
        SD = ShowDefects("Test Defects", self.defects, self)
        rc = SD.run()
        if rc == Gtk.ResponseType.OK:
            self.defects = SD.defects

        SD.hide()
        SD.destroy()

        if len(self.defects) > 0:
            self.btn_defects.set_label("Defects ({})".format(len(self.defects)))

    def upload_test_gui(self, *args):
        """Uploads test and attachments."""
        self.upload_test()

    def upload_test(self):
        """Uploads test and attachments."""
        if self.data is None:
            self.write_message("No data available to upload\n")
            return

        self.data["comments"] = self.comments
        self.data["defects"] = self.defects

        rc = ITkDButils.upload_test(self.session, self.data, self.attachments, check_runNumber=True)
        if rc:
            ipos = rc.find("The following details may help:")
            msg = rc[ipos:]
            dbGtkUtils.complain("Failed uploading test", msg)

        else:
            self.write_message("Upload successfull\n")
            dbGtkUtils.ask_for_confirmation("Upload successfull", "")


def main():
    """Main entry."""
    HELP_LINK="https://itkdb-gtk.docs.cern.ch/uploadSingleTest.html"
    parser = argparse.ArgumentParser()

    parser.add_argument("--test-file", help="Name of json file with test data")
    parser.add_argument("--component-id", help="Override component code")
    parser.add_argument("--raw_data", help="Raw data file", default=None)
    parser.add_argument("--attachment", help="Attachment to upload with the test", default=None)
    parser.add_argument("--attach_title", default=None, type=str, help="The attachment description")
    parser.add_argument("--attach_desc", default="", type=str, help="The attachment description")
    parser.add_argument("--verbose", action="store_true", help="Print what's being sent and received")

    args = parser.parse_args()

    # DB login
    dlg = ITkDBlogin.ITkDBlogin()
    client = dlg.get_client()
    if client is None:
        print("Could not connect to DB with provided credentials.")
        dlg.die()
        sys.exit()

    client.user_gui = dlg

    # Start GUI
    UpT = UploadTest(client,
                     payload=args.test_file,
                     help_link=HELP_LINK,
                     attachment=ITkDButils.Attachment(path=args.attachment,
                                                      title=args.attach_title,
                                                      desc=args.attach_desc))

    if gtk_runs:
        UpT.present()
        UpT.connect("destroy", Gtk.main_quit)
        try:
            Gtk.main()

        except KeyboardInterrupt:
            print("Arrrgggg!!!")

    else:
        # Think
        pass

    dlg.die()


if __name__ == "__main__":
    main()



import sys

try:
    import itkdb_gtk
    
except ImportError:
    from pathlib import Path
    cwd = Path(__file__).parent.parent
    sys.path.append(cwd.as_posix())

from itkdb_gtk import ITkDBlogin, ITkDButils

dlg = ITkDBlogin.ITkDBlogin()
client = dlg.get_client()
if client is None:
    print("Could not connect to DB with provided credentials.")
    dlg.die()
    sys.exit()

client.user_gui = dlg

ITkDButils.get_DB_component(client, "PCPTB02")

dlg.die()


#!/usr/bin/env python3
"""A set of utilities for teh warp scanner."""
import pathlib
import serial
import serial.tools.list_ports as list_ports

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import GLib

class QRScanner:
    """Contains information to detect the scanner."""
    def __init__(self, callback):
        self.reader = None
        self.callback = callback
        self.timer_id = None
        self.source_id = None
        self.init_scanner()

    def init_scanner(self):
        """Sets the scanner."""
        self.setup_scanner()
        if self.reader is None:
            self.timer_id = GLib.timeout_add(500, self.find_scanner)
        else:
            print("Found scanner in {}".format(self.reader.name))

    def find_scanner(self, *args):
        """Check if the scanner is there."""
        # if the reader is there, stop the timeput
        if self.reader is not None:
            return False

        # Try to setup the scanner
        self.setup_scanner()
        if self.reader is not None:
            self.timer_id = None
            print("Found scanner in {}".format(self.reader.name))
            return False
        else:
            return True


    def setup_scanner(self):
        """Setup scanner and callback function."""
        if self.reader is not None:
            return

        self.reader = None
        device_signature = '05f9:4204|1a86:7523'
        candidates = list(list_ports.grep(device_signature))
        if not candidates:
            return

        self.reader = serial.Serial(candidates[0].device, 9600)
        self.source_id = GLib.unix_fd_add_full(
                GLib.PRIORITY_DEFAULT,
                self.reader.fileno(),
                GLib.IOCondition.IN | GLib.IOCondition.ERR | GLib.IOCondition.NVAL,
                self.get_line,
                self.callback,
                )


    def finish_timeout(self):
        """Finishes the timeout."""
        GLib.source_remove(self.source_id)
        self.source_id = None

        self.reader.close()
        self.reader = None

        # re-start the search of the scanner
        self.timer_id = GLib.timeout_add(500, self.find_scanner)

    def get_line(self, fd, state, callback):
        """Has to info to read."""
        if state == GLib.IOCondition.IN:
            try:
                available = self.reader.in_waiting
                while True:
                    delta = self.reader.in_waiting - available
                    if not delta:
                        break

                # Get data from serial device passed in via
                data = self.reader.read_until(expected='\r', size=self.reader.in_waiting).strip()
                txt = data.decode('utf-8')
                if callback:
                    callback(txt)

            except OSError:
                if not pathlib.Path(self.reader.name).exists():
                    print("Device unplugged.")
                    self.finish_timeout()
                    return False

            return True


        else:
            self.finish_timeout()
            return False

def get_text_from_scanner(txt):
    """Callback where the scanners sends the text."""
    print("### {}".format(txt))

def test_scanner():
    """Test the thing."""
    scanner = QRScanner(get_text_from_scanner)
    
    loop = GLib.MainLoop()
    try:
        loop.run()
    except KeyboardInterrupt:
        loop.quit()

if __name__ == "__main__":
    test_scanner()
